% Function, used to solve for "shadow prices"
% Based on the work of Tom Holden's paper: http://www.dynare.org/wp-repo/dynarewp004.pdf
% Michael Paetz, November 15, 2011

function solve_shadowfun(TShadow,TIRF)

Options = optimset( 'MaxIter', 4096, 'LargeScale', 'off' );
ZeroVec = zeros( TShadow, 1 );

gIShadow = zeros( TIRF, TShadow );

		SN = 'pi';

		alpha = ZeroVec;


			gI = GetIRF( 'i', SN, TIRF );

			for indx = 0:( TShadow - 1 )
				gIShadow( :, indx + 1 ) = GetRelIRF( 'i', [ 'shadowI_' sprintf( '%d', indx ) ]);
			end

			% alpha' * ( gI( 1:TShadow ) + gIShadow( 1:TShadow, : ) * alpha ) = 0
			% => 1/2 * alpha' * ( gIShadow( 1:TShadow, : ) + gIShadow( 1:TShadow, : )' ) * alpha + gI' * alpha = 0
			% gI + gIShadow * alpha >= 0
			% => -gIShadow * alpha <= gI

			[ alpha, FoundValue ] =  quadprog( gIShadow( 1:TShadow, : ) + gIShadow( 1:TShadow, : )', gI( 1:TShadow ), -gIShadow, gI, [], [], ZeroVec, [], alpha, Options );

			% gI = gI + gIShadow * alpha;

			if FoundValue < 1e-16
				fprintf( 1, [ 'Solved problem with ' SN ' shock.' ], sum( alpha > 1e-16 ) );

            y_bound = GetBoundedIRF('y',SN,alpha,TIRF);
            pi_bound = GetBoundedIRF('pi',SN,alpha,TIRF);
            i_bound = GetBoundedIRF('i',SN,alpha,TIRF);
            
            y_unbound = GetIRF('y',SN,TIRF);
            pi_unbound = GetIRF('pi',SN,TIRF);
            i_unbound = GetIRF('i',SN,TIRF);

			else
				error( 'JMP:NoSolution', [ 'Failed to solve problem with ' SN ' shock. Increase number of periods, after which you believe the constraint will no longer bind!' ]);
            end
		    
            xx=(0:TIRF-1)';
            
            figure;
            subplot(2,2,1)
            plot(xx,y_unbound(1:TIRF),'-','MarkerSize',7,'LineWidth',1.3);
            hold on;
            plot(xx,GetMean('y')*ones(TIRF),'-','MarkerSize',3,'LineWidth',1.3);
            plot(xx,y_bound(1:TIRF),'--','MarkerSize',7,'LineWidth',1.3);
            title('Output Gap','fontsize',11);
            hold off;
            subplot(2,2,2)
            plot(xx,pi_unbound(1:TIRF),'-','MarkerSize',7,'LineWidth',1.3);
            hold on;
            plot(xx,GetMean('pi')*ones(TIRF),'-','MarkerSize',3,'LineWidth',1.3);
            plot(xx,pi_bound(1:TIRF),'--','MarkerSize',7,'LineWidth',1.3);
            title('Inflation','fontsize',11);  
            hold off;
            subplot(2,2,3)
            plot(xx,i_unbound(1:TIRF),'-','MarkerSize',7,'LineWidth',1.3);
            hold on;
            plot(xx,GetMean('i')*ones(TIRF),'-','MarkerSize',3,'LineWidth',1.3);
            plot(xx,i_bound(1:TIRF),'--','MarkerSize',7,'LineWidth',1.3);
            title('Interest Rate','fontsize',11);  
            
          

